@@ -2,15 +2,16 @@
 time_series_predictor script
 """
 import queue
+import tempfile
 import threading
 import warnings
 
 import psutil
 import torch
-import tempfile
 from skorch import NeuralNetRegressor
-from skorch.callbacks import Callback, EarlyStopping, Checkpoint
+from skorch.callbacks import Callback, Checkpoint, EarlyStopping
 from time_series_dataset import TimeSeriesDataset
+from IPython.utils import io
 
 from sklearn.pipeline import Pipeline
 
@@ -20,6 +21,21 @@ from .sklearn import TransformedTargetRegressor, sample_predict
 # Show switch to cpu warning
 warnings.filterwarnings("default", category=ResourceWarning)
 
+cp = {}
+
+class CheckpointHandler(Callback):
+    """CheckpointHandler
+
+    """
+
+    def on_train_end(self, net, X=None, y=None, **kwargs):
+        with io.capture_output() as _:
+            net.initialize()
+            net.set_params(module__input_dim=X.shape[-1],
+                        module__output_dim=y.shape[-1])
+        net.load_params(
+            checkpoint=cp['checkpoint'])
+
 class InputShapeSetter(Callback):
     """InputShapeSetter
     dynamically set the input size of the PyTorch module based on the data
@@ -30,9 +46,10 @@ class InputShapeSetter(Callback):
     you cannot know the number of features in advance. The best solution would be to set
     the input size dynamically.
     """
+
     def on_train_begin(self, net, X=None, y=None, **kwargs):
         net.set_params(module__input_dim=X.shape[-1],
-                    module__output_dim=y.shape[-1])
+                       module__output_dim=y.shape[-1])
 
 def _get_nnr(net, callbacks, **neural_net_regressor_params):
     return NeuralNetRegressor(
@@ -41,6 +58,7 @@ def _get_nnr(net, callbacks, **neural_net_regressor_params):
         **neural_net_regressor_params
     )
 
+
 class TimeSeriesPredictor:
     """Network agnostic time series predictor class
 
@@ -55,38 +73,44 @@ class TimeSeriesPredictor:
         self.dataset = None
         self.early_stopping = early_stopping
         self.cpu_count = psutil.cpu_count(logical=True)
-        self.checkpoint = Checkpoint(dirname=tempfile.gettempdir())
+        cp['checkpoint'] = Checkpoint(dirname=tempfile.gettempdir())
         if 'train_split' in neural_net_regressor_params:
             train_split = neural_net_regressor_params.get('train_split')
             if train_split is None:
-                self.checkpoint.monitor = 'train_loss_best'
+                cp['checkpoint'].monitor = 'train_loss_best'
                 if early_stopping is not None:
                     if early_stopping.monitor is 'valid_loss':
-                        raise ValueError('Select a valid train_split or disable early_stopping! A valid train_split needs to be selected when valid_loss monitor is selected as early stopping criteria.')
+                        raise ValueError(
+                            'Select a valid train_split or disable early_stopping! A valid train_split needs to be selected when valid_loss monitor is selected as early stopping criteria.')
         if 'device' in neural_net_regressor_params:
             device = neural_net_regressor_params.get('device')
         else:
             device = 'cuda' if torch.cuda.is_available() else 'cpu'
-        neural_net_regressor_params.setdefault('device', device)
+        neural_net_regressor_params['device'] = device
 
+        self.neural_net_regressor_params = neural_net_regressor_params
         self.ttr = TransformedTargetRegressor(
-            regressor=Pipeline(
-                [
-                    ('input scaler', Scaler()),
-                    ('regressor', _get_nnr(
-                        net,
-                        [
-                            InputShapeSetter(),
-                            early_stopping,
-                            self.checkpoint
-                        ],
-                        **neural_net_regressor_params))
-                ]
-            ),
+            regressor=self._get_pipeline(net, cp['checkpoint']),
             transformer=Scaler()
         )
 
-        self.neural_net_regressor_params = neural_net_regressor_params
+    def _get_pipeline(self, net, _checkpoint):
+        return Pipeline(
+            [
+                ('input scaler', Scaler()),
+                ('regressor', _get_nnr(
+                        net,
+                        [
+                            InputShapeSetter(),
+                            self.early_stopping,
+                            _checkpoint,
+                            CheckpointHandler()
+                        ],
+                        **self.neural_net_regressor_params
+                    )
+                )
+            ]
+        )
 
     def make_future_dataframe(self, *args, **kwargs):
         """make_future_dataframe
@@ -161,7 +185,7 @@ class TimeSeriesPredictor:
         **fit_params: dict
           Additional parameters passed to the forward method of the module and to the
           self.train_split call.
-        
+
         """
         print(f"Using device {self.neural_net_regressor_params.get('device')}")
         self.dataset = dataset
@@ -172,21 +196,11 @@ class TimeSeriesPredictor:
                 warnings.warn(
                     '\nSwitching device to cpu to workaround CUDA out of memory problem.',
                     ResourceWarning)
-                self.neural_net_regressor_params.setdefault('device', 'cpu')
-                self.ttr.regressor['regressor'] = _get_nnr(
-                    self.ttr.regressor['regressor'].module,
-                    [
-                        InputShapeSetter(),
-                        self.early_stopping,
-                        self.checkpoint
-                    ],
-                    **self.neural_net_regressor_params
-                )
+                self.neural_net_regressor_params['device'] = 'cpu'
+                self.ttr.regressor = self._get_pipeline(self.ttr.regressor['regressor'].module, cp['checkpoint'])
                 self.ttr.fit(dataset.x, dataset.y, **fit_params)
             else:
                 raise
-            self.ttr.regressor['regressor'].initialize()
-            self.ttr.regressor['regressor'].load_params(checkpoint=self.checkpoint)
 
     def score(self, dataset):
         """Compute the mean r2_score of a network on a given dataset.
@@ -228,10 +242,12 @@ class TimeSeriesPredictor:
         device = self.neural_net_regressor_params.get('device')
         return torch.mean(torch.Tensor(losses).to(device)).cpu().numpy().take(0)
 
+
 class ScoreCalculator(threading.Thread):
     """ScoreCalculator
     Spread computational effort across cpus
     """
+
     def __init__(self, scores_to_calculate, locks, score_method, output_list):
         super().__init__()
         # print("Web Crawler Worker Started: {}".format(threading.current_thread()))
